/*
 * ELF payload decoding support.
 *
 * Copyright (C) 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <exec/segment.h>



/* Generic program segment interface. */

template <typename PROGRAM_HEADER>
class ProgramSegmentVariant : public Segment
{
protected:
  PROGRAM_HEADER *_header;

public:
  explicit ProgramSegmentVariant(PROGRAM_HEADER *header);

  bool loadable();
  offset_t file_contents();
  offset_t file_offset();
  l4_addr_t region_address();
  offset_t region_size();
  l4re_rm_flags_t region_flags();
};



/* Generic interface for an ELF payload. */

class Payload
{
protected:
  Segment **_segments = NULL;

public:
  virtual ~Payload();

  virtual l4_addr_t entry_point() = 0;
  virtual offset_t header_extent() = 0;
  virtual offset_t program_header_extent() = 0;
  virtual unsigned int segments();
  virtual Segment *segment(unsigned int i) = 0;
};

template <typename HEADER, typename PROGRAM_HEADER>
class PayloadVariant : public Payload
{
protected:
  HEADER *_header;

public:
  explicit PayloadVariant(HEADER *header);

  l4_addr_t entry_point();
  offset_t header_extent();
  offset_t program_header_extent();
  unsigned int segments();
  Segment *segment(unsigned int i);
};



/* Return the appropriate payload object. */

Payload *get_payload(char *buf);

/* vim: tabstop=2 expandtab shiftwidth=2
*/
